#!/usr/bin/env perl 

#------------------------------------------------------------------------------
# 
# 1. DISCLAIMER
#    THE SOFTWARE AND/OR RELATED MATERIALS ARE PROVIDED "AS-IS" WITHOUT
#    WARRANTY OF ANY KIND INCLUDING ANY WARRANTIES OF PERFORMANCE OR
#    MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE OR PURPOSE (AS SET FORTH
#    IN UCC 2312-2313) OR FOR ANY PURPOSE WHATSOEVER, FOR THE LICENSED
#    PRODUCT, HOWEVER USED.
#
#    IN NO EVENT SHALL CALTECH/JPL BE LIABLE FOR ANY DAMAGES AND/OR COSTS,
#    INCLUDING BUT NOT LIMITED TO INCIDENTAL OR CONSEQUENTIAL DAMAGES OF ANY
#    KIND, INCLUDING ECONOMIC DAMAGE OR INJURY TO PROPERTY AND LOST PROFITS,
#    REGARDLESS OF WHETHER CIT/JPL SHALL BE ADVISED, HAVE REASON TO KNOW,
#    OR IN FACT SHALL KNOW OF THE POSSIBILITY.
#
#    USER BEARS ALL RISK RELATING TO QUALITY AND PERFORMANCE OF THE SOFTWARE
#    AND/OR RELATED MATERIALS.
#
# 2. The user shall comply with all applicable U.S. export control laws and
#    regulations.  To the extent that the software is subject to U.S. export
#    control laws and regulations, the user has the responsibility to obtain
#    export licenses or other export authority as may be required before
#    exporting such information to foreign countries or providing access to
#    foreign persons.
#
#------------------------------------------------------------------------------


use strict;
use vars qw ($opt_s $opt_h $opt_m $opt_r);
use Getopt::Std;
use File::Basename;


my $Usage = "Usage: $0 -h -s attitude_file_1 [attitude_file_2]\n"
           ."    This program will display the attitude file\n"
           ."    -h option will display the program usage\n"
           ."    -s option will display only the header record\n"
           #."    -r option will reverse the bytes \n"
           ."    -m option will generate the .met file\n" 
           ."    if second file exists, both files will be compared\n";

#Parse the command line
getopts('hsmr');

die $Usage if defined $opt_h || $#ARGV+1 < 1;

my $is_little_endian = unpack('c',pack('s',1));
#my $is_little_endian = defined $opt_r ? 1 : 0;

my $file_hdr_format= 'A24A48A32A8ddfLLLLLf16f4A280';      
my $file_rec_format= 'dddddddLA4';
my $file_rec2_format= 'LA4ddd';
my @REC_OUT_format = ( "DateTime(secTAI93)= ",
                       "Euler angle (rad) = ", 
                       "Ang rate(rad/sec) = ",
                       "Attitude quality  = ");
my @REC_OUT2_format= ( "Orbit number                                    = ",
                       "Time upward TOD equator crossing (secTAI93)     = ",
                       "Time downward TOD equator crossing (secTAI93)   = ",
                       "Obit down-cross terrestrial longitude (radians) = ");
my @REC_OUT_HDR  = (   "Spacecraft Name = ",
                       "Start and Stop times = ",
                       "Source of the  attitude data                     = ",
                       "Version Number                                   = ",
                       "Attitude  dataset start time (secTAI93)          = ",
                       "Attitude  dataset end   time (secTAI93)          = ",
                       "Expected interval betweeen records               = ",
                       "Number of input dataset univeral references      = ",
                       "Number of attitude  records                      = ",
                       "Order of rotations as permutation of 1=x,2=y,3=z = ",
                       "Attitude data quality processing parameters      = ",
                       "Quality assurance statistics                     = "
                   );
my $DATA_SZ = 64;
my $MET_SZ  = 32;


my $filen = $ARGV[0];
my $filen2 = $ARGV[1] || $ARGV[0];
open(INFILE,$filen) or die "Can not open the file $filen : $?\n";
binmode(INFILE);
open(INFILE2,$filen2) or die "Can not open the file $filen2 : $?\n";
binmode(INFILE2);

# Read in the Attitude Header record
my $hdr_rec;
my $hdr_rec2;
read(INFILE,$hdr_rec,512) == 512 
     or die "can not read in the Attitude header record\n";
read(INFILE2,$hdr_rec2,512) == 512 
     or die "can not read in the Attitude header record\n";

$hdr_rec = update_buf($file_hdr_format,$hdr_rec) if $is_little_endian == 1;
my @FIELDS = unpack($file_hdr_format,$hdr_rec);

$hdr_rec2 = update_buf($file_hdr_format,$hdr_rec2) if $is_little_endian == 1;
my @FIELDS2 = unpack($file_hdr_format,$hdr_rec2);

dump_eph_header( $filen,$filen2,\@FIELDS,\@FIELDS2);

&create_met($filen,\@FIELDS) if defined $opt_m;

exit if defined $opt_s || defined $opt_m;

my $n_ur_rec = $FIELDS[7];
my $n_data_rec = $FIELDS[8];
my $n_met_rec  = $FIELDS[9];

# Read in Universal Reference record
my $ur_rec;
my $ur_rec2;
for(my $i=0; $i < $n_ur_rec; $i++) {
   read(INFILE,$ur_rec,256) == 256 
        or die "can not read in the Universal Ref records\n";
   read(INFILE2,$ur_rec2,256) == 256 
        or die "can not read in the Universal Ref records\n";
}

# Read in Attitude Record 
my $data_rec;
my $data_rec2;
for(my $i=0; $i < $n_data_rec; $i++) {
   read(INFILE,$data_rec,$DATA_SZ) == $DATA_SZ 
        or die "can not read in Attitude record at $i\n";
   $data_rec=update_buf($file_rec_format,$data_rec) if $is_little_endian==1;
   read(INFILE2,$data_rec2,$DATA_SZ) == $DATA_SZ 
        or die "can not read in Attitude record at $i\n";
   $data_rec2=update_buf($file_rec_format,$data_rec2) if $is_little_endian==1;
   my ($secTai,$quality,$dy);
   my @vec1 = (0.0,0.0,0.0);
   my @vec2 = (0.0,0.0,0.0);
   ($secTai,$vec1[0],$vec1[1],$vec1[2],
    $vec2[0],$vec2[1],$vec2[2],$quality,$dy) 
              = unpack($file_rec_format,$data_rec);
   my ($secTai2,$quality2,$dy2);
   my @vec1_2 = (0.0,0.0,0.0);
   my @vec2_2 = (0.0,0.0,0.0);
   ($secTai2,$vec1_2[0],$vec1_2[1],$vec1_2[2],
    $vec2_2[0],$vec2_2[1],$vec2_2[2],$quality2,$dy2) 
              = unpack($file_rec_format,$data_rec2);
   print "\n";
   xprint ($REC_OUT_format[0],$secTai,$secTai2);
   xprint ($REC_OUT_format[1],join(" ",@vec1),join(" ",@vec1_2));
   xprint ($REC_OUT_format[2],join(" ",@vec2),join(" ",@vec2_2));
   xprint ($REC_OUT_format[3],$quality,$quality2);
}
# End of program

sub update_buf {
   my ($format,$buf) = @_;
   my %SZ = ( 'A'=> 1, 'd'=> 8, 'f' => 4, 'N' => 4, 'n' => 4, 'L' => 4);
   my @fields = ($format =~ m/([AdfNL]\d*)/g);
   my $iloc = 0;
   my ($char_f,$dig,$repeat);
   foreach (my $i=0; $i <= $#fields; $i++) {
      ($char_f,$dig) = ($fields[$i] =~ m/([A-Z,a-z])(\d*)/g);
      $repeat = $dig || 1;
      if ( $char_f =~ /(d|f|L)/ ) {
         $buf = swap($buf,$iloc,$repeat,$SZ{$char_f});
      }
      # Update the pointer
      $iloc += ($repeat) * ($SZ{$char_f}) ;
   }
   return $buf;
}

sub swap {
   my ($buf,$b_offset,$ntimes,$nsize) = @_;
   my $byte_offset = $b_offset;
   my $buffer = $buf;
   foreach (my $i=0; $i < $ntimes; $i++) {
      my $str = substr($buffer,$byte_offset,$nsize);
      substr($buffer,$byte_offset,$nsize) = reverse $str;
      $byte_offset += $nsize;
   }
   return $buffer;
}

sub dump_eph_header {
   my ($filen,$filen2,$arrayref1,$arrayref2) = @_;
   my @FIELDS = @$arrayref1;
   my @FIELDS2 = @$arrayref2;
   print "Input file = $filen";
   print "--> $filen2" if $filen ne $filen2;
   print "\n\n";
   print "Header information \n\n";
   for (my $i=0; $i < 9; $i++) {
      xprint ($REC_OUT_HDR[$i],$FIELDS[$i],$FIELDS2[$i]);
   }
   xprint ($REC_OUT_HDR[9],join(" ",@FIELDS[9..11]),
                           join(" ",@FIELDS2[9..11]));
   xprint ($REC_OUT_HDR[10],join(" ",@FIELDS[12..27]),
                            join(" ",@FIELDS2[12..27]));
   xprint ($REC_OUT_HDR[11],join(" ",@FIELDS[28..31]),
                            join(" ",@FIELDS2[28..31]));
   print "\n\n"; 
}

sub xprint {
   my ($prt_str,$data1,$data2) = @_;
   print "$prt_str$data1\n";
   print ">$prt_str$data2\n" if $data1 cmp $data2;
}


sub create_met {
   use Time::localtime;
   my ($file,$arrayref) = @_;
   my @FIELDS = @$arrayref;
   my $metfile = $file . ".met";
   my $SizeMBECSDataGranule = (-s $file)/1048576.0;
   my $LocalGranuleID = basename($file);
   my $tm = localtime;
   my $ProductionDateTime = sprintf ("%04d-%02d-%02dT%02d:%02d:%02d.00000Z",
       ($tm->year)+1900,($tm->mon)+1,$tm->mday,$tm->hour,$tm->min,$tm->sec);
   my $end_Time = `run_TAIandUTC $FIELDS[5]`;
   my ($RangeEndingDate,$RangeEndingTime) = 
      ($end_Time =~ m/(\d\d\d\d-\d\d-\d\d)T(\d\d:\d\d:\d\d.\d+)/ );
   my $begin_Time = `run_TAIandUTC $FIELDS[4]`;
   my ($RangeBeginningDate,$RangeBeginningTime) = 
      ($begin_Time =~ m/(\d\d\d\d-\d\d-\d\d)T(\d\d:\d\d:\d\d.\d+)/ );
   
   print "DEBUG: RangeBeginningDate = $RangeBeginningDate\n";
   print "DEBUG: RangeBeginningTime = $RangeBeginningTime\n";
   print "DEBUG: RangeEndingDate = $RangeEndingDate\n";
   print "DEBUG: RangeEndingTime = $RangeEndingTime\n";
   print "DEBUG: ProductionDateTime = $ProductionDateTime\n";
   print "DEBUG: LocalGranuleID = $LocalGranuleID\n";
   print "DEBUG: SizeMBECSDataGranule = $SizeMBECSDataGranule\n";

   #my $RangeEndingTime
   #my $RangeEndingDate
   #my $RangeBeginningTime
   #my $RangeBeginningDate

   my $met_content = <<"EOF";
GROUP = INVENTORYMETADATA
GROUPTYPE = MASTERGROUP
    GROUP = CollectionDescriptionClass
        OBJECT = ShortName
            Value = "PM1ATTNR"
            TYPE = "STRING"
            NUM_VAL = 1
        END_OBJECT = ShortName
        OBJECT = VersionID
            Value = 1
            TYPE = "INTEGER"
            NUM_VAL = 1
        END_OBJECT = VersionID
    END_GROUP = CollectionDescriptionClass
    GROUP = ECSDataGranule
        OBJECT = SizeMBECSDataGranule
            Value = $SizeMBECSDataGranule
            TYPE = "DOUBLE"
            NUM_VAL = 1
        END_OBJECT = SizeMBECSDataGranule
        OBJECT = ReprocessingPlanned
            Value = "no further update anticipated"
            TYPE = "STRING"
            NUM_VAL = 1
        END_OBJECT = ReprocessingPlanned
        OBJECT = ReprocessingActual
            Value = "processed once"
            TYPE = "STRING"
            NUM_VAL = 1
        END_OBJECT = ReprocessingActual
        OBJECT = LocalGranuleID
            Value = "$LocalGranuleID"
            TYPE = "STRING"
            NUM_VAL = 1
        END_OBJECT = LocalGranuleID
        OBJECT = ProductionDateTime
            Value = "$ProductionDateTime"
            TYPE = "TIME"
            NUM_VAL = 1
        END_OBJECT = ProductionDateTime
    END_GROUP = ECSDataGranule
    GROUP = PGEVersionClass
    END_GROUP = PGEVersionClass
    GROUP = RangeDateTime
        OBJECT = RangeEndingTime
            Value = "$RangeEndingTime"
            TYPE = "STRING"
            NUM_VAL = 1
        END_OBJECT = RangeEndingTime
        OBJECT = RangeEndingDate
            Value = "$RangeEndingDate"
            TYPE = "DATE"
            NUM_VAL = 1
        END_OBJECT = RangeEndingDate
        OBJECT = RangeBeginningTime
            Value = "$RangeBeginningTime"
            TYPE = "STRING"
            NUM_VAL = 1
        END_OBJECT = RangeBeginningTime
        OBJECT = RangeBeginningDate
            Value = "$RangeBeginningDate"
            TYPE = "DATE"
            NUM_VAL = 1
        END_OBJECT = RangeBeginningDate
    END_GROUP = RangeDateTime
    GROUP = SpatialDomainContainer
        GROUP = HorizontalSpatialDomainContainer
            GROUP = ZoneIdentifierClass
            END_GROUP = ZoneIdentifierClass
        END_GROUP = HorizontalSpatialDomainContainer
    END_GROUP = SpatialDomainContainer
    GROUP = MeasuredParameter
        OBJECT = MeasuredParameterContainer
            CLASS = "1"
            OBJECT = ParameterName
                CLASS = "1"
                Value = "Attitude"
                TYPE = "STRING"
                NUM_VAL = 1
            END_OBJECT = ParameterName
            GROUP = QAStats
                CLASS = "1"
                OBJECT = QAPercentMissingData
                    CLASS = "1"
                    Value = 1
                    TYPE = "INTEGER"
                    NUM_VAL = 1
                END_OBJECT = QAPercentMissingData
                OBJECT = QAPercentOutofBoundsData
                    CLASS = "1"
                    Value = 1
                    TYPE = "INTEGER"
                    NUM_VAL = 1
                END_OBJECT = QAPercentOutofBoundsData
                OBJECT = QAPercentInterpolatedData
                    CLASS = "1"
                    Value = 0
                    TYPE = "INTEGER"
                    NUM_VAL = 1
                END_OBJECT = QAPercentInterpolatedData
            END_GROUP = QAStats
            GROUP = QAFlags
                CLASS = "1"
                OBJECT = AutomaticQualityFlag
                    CLASS = "1"
                    Value = "Failed"
                    TYPE = "STRING"
                    NUM_VAL = 1
                END_OBJECT = AutomaticQualityFlag
                OBJECT = AutomaticQualityFlagExplanation
                    CLASS = "1"
                    Value = "Long gap/range violation in attitude time-line."
                    TYPE = "STRING"
                    NUM_VAL = 1
                END_OBJECT = AutomaticQualityFlagExplanation
            END_GROUP = QAFlags
        END_OBJECT = MeasuredParameterContainer
    END_GROUP = MeasuredParameter
    GROUP = AdditionalAttributes
        OBJECT = AdditionalAttributesContainer
            CLASS = "1"
            OBJECT = AdditionalAttributeName
                CLASS = "1"
                Value = "QAPercentBadEphemerisData"
                TYPE = "STRING"
                NUM_VAL = 1
            END_OBJECT = AdditionalAttributeName
            GROUP = InformationContent
                CLASS = "1"
                OBJECT = ParameterValue
                    CLASS = "1"
                    Value = ("0")
                    TYPE = "STRING"
                    NUM_VAL = 1
                END_OBJECT = ParameterValue
            END_GROUP = InformationContent
        END_OBJECT = AdditionalAttributesContainer
        OBJECT = AdditionalAttributesContainer
            CLASS = "2"
            OBJECT = AdditionalAttributeName
                CLASS = "2"
                Value = "QAPercentBadStatusWords"
                TYPE = "STRING"
                NUM_VAL = 1
            END_OBJECT = AdditionalAttributeName
            GROUP = InformationContent
                CLASS = "2"
                OBJECT = ParameterValue
                    CLASS = "2"
                    Value = ("0")
                    TYPE = "STRING"
                    NUM_VAL = 1
                END_OBJECT = ParameterValue
            END_GROUP = InformationContent
        END_OBJECT = AdditionalAttributesContainer
        OBJECT = AdditionalAttributesContainer
            CLASS = "3"
            OBJECT = AdditionalAttributeName
                CLASS = "3"
                Value = "QAPercentMissingStatusWords"
                TYPE = "STRING"
                NUM_VAL = 1
            END_OBJECT = AdditionalAttributeName
            GROUP = InformationContent
                CLASS = "3"
                OBJECT = ParameterValue
                    CLASS = "3"
                    Value = ("0")
                    TYPE = "STRING"
                    NUM_VAL = 1
                END_OBJECT = ParameterValue
            END_GROUP = InformationContent
        END_OBJECT = AdditionalAttributesContainer
    END_GROUP = AdditionalAttributes
    GROUP = InputGranule
        OBJECT = InputPointer
            Value = ("\nLGID:PM1EPHND:001:PM1EPHND.P2002304.1200.001.2002305122228", "LGID:PM1EPHND:001:PM1EPHND.P2002303.1200.001.2002304125733", "UR:10:DsShESDTUR:UR:15:DsShSciServerUR:13:[GSF:DSSDSRV]:24:SC:PMCOGBAD.001:17179717", "UR:10:DsShESDTUR:UR:15:DsShSciServerUR:13:[GSF:DSSDSRV]:24:SC:PMCOGBAD.001:17179718")
            TYPE = "STRING"
            NUM_VAL = 4
        END_OBJECT = InputPointer
    END_GROUP = InputGranule
END_GROUP = INVENTORYMETADATA
GROUP = COLLECTIONMETADATA
GROUPTYPE = MASTERGROUP
   OBJECT = DLLName
      Mandatory = "TRUE"
      Data_Location = "MCF"
      NUM_VAL = 1
      TYPE = "STRING"
      Value = "libDsESDTSyBASIC.001Sh.so"
   END_OBJECT = DLLName
   OBJECT = spatialSearchType
      Mandatory = "TRUE"
      Data_Location = "MCF"
      NUM_VAL = 1
      TYPE = "STRING"
      Value = "NotSupported"
   END_OBJECT = spatialSearchType
   GROUP = CollectionDescriptionClass
      OBJECT = VersionID
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "INTEGER"
         Value = 1
      END_OBJECT = VersionID
      OBJECT = LongName
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "Preprocessed Aqua Platform Refined Attitude Data in Native format"
      END_OBJECT = LongName
      OBJECT = ShortName
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "PM1ATTNR"
      END_OBJECT = ShortName
      OBJECT = CollectionDescription
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "This collection consists of Aqua Platform Refined Attitude Data that has been pre-processed by ECS to an internal standard supported by the ECS SDP Toolkit"
      END_OBJECT = CollectionDescription
   END_GROUP = CollectionDescriptionClass
   GROUP = ECSCollection
      OBJECT = VersionDescription
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "Initial Version"
      END_OBJECT = VersionDescription
      OBJECT = ProcessingCenter
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "GSFC"
      END_OBJECT = ProcessingCenter
      OBJECT = ArchiveCenter
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "GSFC"
      END_OBJECT = ArchiveCenter
      OBJECT = SuggestedUsage
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "This data is typically used in determining the geolocation of earth remote sensing observations."
      END_OBJECT = SuggestedUsage
      OBJECT = RevisionDate
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "DATE"
         Value = 2001-07-18
      END_OBJECT = RevisionDate
   END_GROUP = ECSCollection
   GROUP = SingleTypeCollection
      OBJECT = AccessConstraints
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "None"
      END_OBJECT = AccessConstraints
      OBJECT = MaintenanceandUpdateFrequency
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "Continually"
      END_OBJECT = MaintenanceandUpdateFrequency
      OBJECT = CollectionState
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "In Work"
      END_OBJECT = CollectionState
   END_GROUP = SingleTypeCollection
   GROUP = Temporal
      GROUP = RangeDateTime
         OBJECT = RangeEndingDate
            Mandatory = "TRUE"
            Data_Location = "MCF"
            NUM_VAL = 1
            TYPE = "DATE"
            Value = 2000-12-21
         END_OBJECT = RangeEndingDate
         OBJECT = RangeBeginningDate
            Mandatory = "TRUE"
            Data_Location = "MCF"
            NUM_VAL = 1
            TYPE = "DATE"
            Value = 2000-12-21
         END_OBJECT = RangeBeginningDate
         OBJECT = RangeBeginningTime
            Mandatory = "TRUE"
            Data_Location = "MCF"
            NUM_VAL = 1
            TYPE = "TIME"
            Value = "00:00:00.00000"
         END_OBJECT = RangeBeginningTime
         OBJECT = RangeEndingTime
            Mandatory = "TRUE"
            Data_Location = "MCF"
            NUM_VAL = 1
            TYPE = "TIME"
            Value = "00:00:00.00000"
         END_OBJECT = RangeEndingTime
      END_GROUP = RangeDateTime
      OBJECT = DateType
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "Gregorian"
      END_OBJECT = DateType
      OBJECT = TimeType
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "UTC"
      END_OBJECT = TimeType
      OBJECT = TemporalRangeType
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "Continuous Range"
      END_OBJECT = TemporalRangeType
      OBJECT = EndsatPresentFlag
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "Y"
      END_OBJECT = EndsatPresentFlag
      OBJECT = PrecisionofSeconds
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "INTEGER"
         Value = 3
      END_OBJECT = PrecisionofSeconds
   END_GROUP = Temporal
   GROUP = ProcessingLevel
      OBJECT = ProcessingLevelDescription
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "Telemetry Data"
      END_OBJECT = ProcessingLevelDescription
      OBJECT = ProcessingLevelID
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "0"
      END_OBJECT = ProcessingLevelID
   END_GROUP = ProcessingLevel
   GROUP = Contact
      GROUP = ContactOrganization
         OBJECT = ContactOrganizationContainer
            Data_Location = "NONE"
            Mandatory = "TRUE"
            CLASS = "1"
            GROUP = OrganizationEmail
               CLASS = "1"
               OBJECT = ElectronicMailAddress
                  Mandatory = "TRUE"
                  Data_Location = "MCF"
                  NUM_VAL = 1
                  TYPE = "STRING"
                  Value = ("daacuso\@daac.gsfc.nasa.gov")
               END_OBJECT = ElectronicMailAddress
            END_GROUP = OrganizationEmail
            GROUP = OrganizationTelephone
               CLASS = "1"
               OBJECT = OrganizationTelephoneContainer
                  Data_Location = "NONE"
                  Mandatory = "TRUE"
                  CLASS = "1"
                  OBJECT = TelephoneNumberType
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "Voice"
                  END_OBJECT = TelephoneNumberType
                  OBJECT = TelephoneNumber
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "301-614-5224"
                  END_OBJECT = TelephoneNumber
               END_OBJECT = OrganizationTelephoneContainer
               OBJECT = OrganizationTelephoneContainer
                  Data_Location = "NONE"
                  Mandatory = "TRUE"
                  CLASS = "2"
                  OBJECT = TelephoneNumberType
                     Mandatory = "TRUE"
                     CLASS = "2"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "Voice"
                  END_OBJECT = TelephoneNumberType
                  OBJECT = TelephoneNumber
                     Mandatory = "TRUE"
                     CLASS = "2"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "1-800-257-6151"
                  END_OBJECT = TelephoneNumber
               END_OBJECT = OrganizationTelephoneContainer
            END_GROUP = OrganizationTelephone
            GROUP = ContactOrganizationAddress
               CLASS = "1"
               OBJECT = ContactOrganizationAddressContainer
                  Data_Location = "NONE"
                  Mandatory = "TRUE"
                  CLASS = "1"
                  OBJECT = Country
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "USA"
                  END_OBJECT = Country
                  OBJECT = StateProvince
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "MD"
                  END_OBJECT = StateProvince
                  OBJECT = StreetAddress
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "NASA Goddard Space Flight Center"
                  END_OBJECT = StreetAddress
                  OBJECT = PostalCode
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "20771"
                  END_OBJECT = PostalCode
                  OBJECT = City
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "Greenbelt"
                  END_OBJECT = City
               END_OBJECT = ContactOrganizationAddressContainer
            END_GROUP = ContactOrganizationAddress
            OBJECT = HoursofService
               Mandatory = "TRUE"
               CLASS = "1"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "9 AM to 5 PM ET"
            END_OBJECT = HoursofService
            OBJECT = Role
               Mandatory = "TRUE"
               CLASS = "1"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "Archive"
            END_OBJECT = Role
            OBJECT = ContactOrganizationName
               Mandatory = "TRUE"
               CLASS = "1"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "GSFC DAAC Helpdesk"
            END_OBJECT = ContactOrganizationName
            OBJECT = ContactInstructions
               Mandatory = "TRUE"
               CLASS = "1"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "None"
            END_OBJECT = ContactInstructions
         END_OBJECT = ContactOrganizationContainer
      END_GROUP = ContactOrganization
      GROUP = ContactPerson
         OBJECT = ContactPersonContainer
            Data_Location = "NONE"
            Mandatory = "TRUE"
            CLASS = "1"
            GROUP = Email
               CLASS = "1"
               OBJECT = ElectronicMailAddress
                  Mandatory = "TRUE"
                  Data_Location = "MCF"
                  NUM_VAL = 1
                  TYPE = "STRING"
                  Value = ("wknauss\@eos.hitc.com")
               END_OBJECT = ElectronicMailAddress
            END_GROUP = Email
            GROUP = Telephone
               CLASS = "1"
               OBJECT = TelephoneContainer
                  Data_Location = "NONE"
                  Mandatory = "TRUE"
                  CLASS = "1"
                  OBJECT = TelephoneNumberType
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "Voice"
                  END_OBJECT = TelephoneNumberType
                  OBJECT = TelephoneNumber
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "301-925-1105"
                  END_OBJECT = TelephoneNumber
               END_OBJECT = TelephoneContainer
               OBJECT = TelephoneContainer
                  Data_Location = "NONE"
                  Mandatory = "TRUE"
                  CLASS = "2"
                  OBJECT = TelephoneNumberType
                     Mandatory = "TRUE"
                     CLASS = "2"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "Facsimile"
                  END_OBJECT = TelephoneNumberType
                  OBJECT = TelephoneNumber
                     Mandatory = "TRUE"
                     CLASS = "2"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "301-925-0321"
                  END_OBJECT = TelephoneNumber
               END_OBJECT = TelephoneContainer
            END_GROUP = Telephone
            GROUP = ContactPersonAddress
               CLASS = "1"
               OBJECT = ContactPersonAddressContainer
                  Data_Location = "NONE"
                  Mandatory = "TRUE"
                  CLASS = "1"
                  OBJECT = StreetAddress
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "1616 McCormick Drive"
                  END_OBJECT = StreetAddress
                  OBJECT = PostalCode
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "20774"
                  END_OBJECT = PostalCode
                  OBJECT = StateProvince
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "MD"
                  END_OBJECT = StateProvince
                  OBJECT = Country
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "USA"
                  END_OBJECT = Country
                  OBJECT = City
                     Mandatory = "TRUE"
                     CLASS = "1"
                     Data_Location = "MCF"
                     NUM_VAL = 1
                     TYPE = "STRING"
                     Value = "Upper Marlboro"
                  END_OBJECT = City
               END_OBJECT = ContactPersonAddressContainer
            END_GROUP = ContactPersonAddress
            OBJECT = ContactFirstName
               Mandatory = "TRUE"
               CLASS = "1"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "William"
            END_OBJECT = ContactFirstName
            OBJECT = HoursofService
               Mandatory = "TRUE"
               CLASS = "1"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "9 AM to 5 PM Eastern, Monday through Friday"
            END_OBJECT = HoursofService
            OBJECT = Role
               Mandatory = "TRUE"
               CLASS = "1"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "Producer"
            END_OBJECT = Role
            OBJECT = ContactJobPosition
               Mandatory = "TRUE"
               CLASS = "1"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "PDPS Engineer for the ECS Release B"
            END_OBJECT = ContactJobPosition
            OBJECT = ContactMiddleName
               Mandatory = "TRUE"
               CLASS = "1"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "unknown"
            END_OBJECT = ContactMiddleName
            OBJECT = ContactLastName
               Mandatory = "TRUE"
               CLASS = "1"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "Knauss"
            END_OBJECT = ContactLastName
            OBJECT = ContactInstructions
               Mandatory = "TRUE"
               CLASS = "1"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "none"
            END_OBJECT = ContactInstructions
         END_OBJECT = ContactPersonContainer
      END_GROUP = ContactPerson
   END_GROUP = Contact
   GROUP = CSDTDescription
      OBJECT = CSDTComments
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "None"
      END_OBJECT = CSDTComments
      OBJECT = PrimaryCSDT
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "Standard Point"
      END_OBJECT = PrimaryCSDT
      OBJECT = IndirectReference
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "None"
      END_OBJECT = IndirectReference
      OBJECT = Implementation
         Mandatory = "TRUE"
         Data_Location = "MCF"
         NUM_VAL = 1
         TYPE = "STRING"
         Value = "Binary"
      END_OBJECT = Implementation
   END_GROUP = CSDTDescription
   GROUP = AdditionalAttributes
      OBJECT = AdditionalAttributesContainer
         Data_Location = "NONE"
         Mandatory = "TRUE"
         CLASS = "1"
         GROUP = PhysicalParameterDetails
            CLASS = "1"
            OBJECT = ParameterMeasurementResolution
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "N/A"
            END_OBJECT = ParameterMeasurementResolution
            OBJECT = ParameterRangeEnd
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "100"
            END_OBJECT = ParameterRangeEnd
            OBJECT = ParameterValueAccuracy
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "1"
            END_OBJECT = ParameterValueAccuracy
            OBJECT = ParameterRangeBegin
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "0"
            END_OBJECT = ParameterRangeBegin
            OBJECT = ParameterValueAccuracyExplanation
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "N/A"
            END_OBJECT = ParameterValueAccuracyExplanation
            OBJECT = ParameterUnitsofMeasurement
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "Percent"
            END_OBJECT = ParameterUnitsofMeasurement
         END_GROUP = PhysicalParameterDetails
         OBJECT = AdditionalAttributeDescription
            Mandatory = "TRUE"
            CLASS = "1"
            Data_Location = "MCF"
            NUM_VAL = 1
            TYPE = "STRING"
            Value = "The percentage of input definitive or predictive ephemeris data points used that had bad quality flag settings"
         END_OBJECT = AdditionalAttributeDescription
         OBJECT = AdditionalAttributeDatatype
            Mandatory = "TRUE"
            CLASS = "1"
            Data_Location = "MCF"
            NUM_VAL = 1
            TYPE = "STRING"
            Value = "int"
         END_OBJECT = AdditionalAttributeDatatype
         OBJECT = AdditionalAttributeName
            Mandatory = "TRUE"
            CLASS = "1"
            Data_Location = "MCF"
            NUM_VAL = 1
            TYPE = "STRING"
            Value = "QAPercentBadEphemerisData"
         END_OBJECT = AdditionalAttributeName
      END_OBJECT = AdditionalAttributesContainer
      OBJECT = AdditionalAttributesContainer
         Data_Location = "NONE"
         Mandatory = "TRUE"
         CLASS = "2"
         GROUP = PhysicalParameterDetails
            CLASS = "2"
            OBJECT = ParameterMeasurementResolution
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "N/A"
            END_OBJECT = ParameterMeasurementResolution
            OBJECT = ParameterRangeEnd
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "100"
            END_OBJECT = ParameterRangeEnd
            OBJECT = ParameterValueAccuracy
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "1"
            END_OBJECT = ParameterValueAccuracy
            OBJECT = ParameterRangeBegin
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "0"
            END_OBJECT = ParameterRangeBegin
            OBJECT = ParameterValueAccuracyExplanation
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "N/A"
            END_OBJECT = ParameterValueAccuracyExplanation
            OBJECT = ParameterUnitsofMeasurement
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "Percent"
            END_OBJECT = ParameterUnitsofMeasurement
         END_GROUP = PhysicalParameterDetails
         OBJECT = AdditionalAttributeDescription
            Mandatory = "TRUE"
            CLASS = "2"
            Data_Location = "MCF"
            NUM_VAL = 1
            TYPE = "STRING"
            Value = "The percentage of records that have a corresponding bad status word in the 8 second interval Ground-Based Attitude Determination granule"
         END_OBJECT = AdditionalAttributeDescription
         OBJECT = AdditionalAttributeDatatype
            Mandatory = "TRUE"
            CLASS = "2"
            Data_Location = "MCF"
            NUM_VAL = 1
            TYPE = "STRING"
            Value = "int"
         END_OBJECT = AdditionalAttributeDatatype
         OBJECT = AdditionalAttributeName
            Mandatory = "TRUE"
            CLASS = "2"
            Data_Location = "MCF"
            NUM_VAL = 1
            TYPE = "STRING"
            Value = "QAPercentBadStatusWords"
         END_OBJECT = AdditionalAttributeName
      END_OBJECT = AdditionalAttributesContainer
      OBJECT = AdditionalAttributesContainer
         Data_Location = "NONE"
         Mandatory = "TRUE"
         CLASS = "3"
         GROUP = PhysicalParameterDetails
            CLASS = "3"
            OBJECT = ParameterMeasurementResolution
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "N/A"
            END_OBJECT = ParameterMeasurementResolution
            OBJECT = ParameterRangeEnd
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "100"
            END_OBJECT = ParameterRangeEnd
            OBJECT = ParameterValueAccuracy
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "1"
            END_OBJECT = ParameterValueAccuracy
            OBJECT = ParameterRangeBegin
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "0"
            END_OBJECT = ParameterRangeBegin
            OBJECT = ParameterValueAccuracyExplanation
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "N/A"
            END_OBJECT = ParameterValueAccuracyExplanation
            OBJECT = ParameterUnitsofMeasurement
               Mandatory = "TRUE"
               Data_Location = "MCF"
               NUM_VAL = 1
               TYPE = "STRING"
               Value = "Percent"
            END_OBJECT = ParameterUnitsofMeasurement
         END_GROUP = PhysicalParameterDetails
         OBJECT = AdditionalAttributeDescription
            Mandatory = "TRUE"
            CLASS = "3"
            Data_Location = "MCF"
            NUM_VAL = 1
            TYPE = "STRING"
            Value = "The percentage of records that are missing a corresponding status word in the 8 second interval Ground-Based Attitude Determination granule"
         END_OBJECT = AdditionalAttributeDescription
         OBJECT = AdditionalAttributeDatatype
            Mandatory = "TRUE"
            CLASS = "3"
            Data_Location = "MCF"
            NUM_VAL = 1
            TYPE = "STRING"
            Value = "int"
         END_OBJECT = AdditionalAttributeDatatype
         OBJECT = AdditionalAttributeName
            Mandatory = "TRUE"
            CLASS = "3"
            Data_Location = "MCF"
            NUM_VAL = 1
            TYPE = "STRING"
            Value = "QAPercentMissingStatusWords"
         END_OBJECT = AdditionalAttributeName
      END_OBJECT = AdditionalAttributesContainer
   END_GROUP = AdditionalAttributes
END_GROUP = COLLECTIONMETADATA
GROUP = ARCHIVEDMETADATA
END_GROUP = ARCHIVEDMETADATA
END
EOF

   open(METFILE,"+>$metfile") or die "Can not open the file $metfile : $!\n";
   print METFILE $met_content;
   close(METFILE);
}
